
package odbdatebugtestcase;

import java.util.Date;
import java.util.List;
import java.util.UUID;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.TypedQuery;
import odbdatebugtestcase.model.Person;

public class DataManager {
    
    private final int GENERATE_PERSONS_COUNT = 1000;
    
    private EntityManagerFactory entityManagerFactory;
    private EntityManager entityManager;
    
    public DataManager(String connectionString) {
        entityManagerFactory = Persistence.createEntityManagerFactory(connectionString);
        entityManager = entityManagerFactory.createEntityManager();
    }
    
    public void initDatabase() {
        long personsCount = getPersonsCount();
        if (personsCount == 0) {
            createPersons(GENERATE_PERSONS_COUNT);
        }
    }
    
    public void updatePersonsBirthday(Date newBirthday) {
        System.out.println("Set new birthday:" + newBirthday.toString());
        try {
            entityManager.getTransaction().begin();
            TypedQuery<Person> personQuery = entityManager.createQuery("select p from Person p", Person.class);
            List<Person> persons = personQuery.getResultList();
            for(Person person : persons) {
                person.setBirthday(newBirthday);
            }
            entityManager.getTransaction().commit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    public void close() {
        entityManager.close();
        entityManagerFactory.close();
    }

    private void createPersons(long count) {
        try {
            entityManager.getTransaction().begin();
            for (int index = 0; index < count; index++) {
                Person person = generatePerson();
                entityManager.persist(person);
            }
            entityManager.getTransaction().commit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    private Person generatePerson() {
        Person person = new Person();
        person.setName(UUID.randomUUID().toString());
        person.setBirthday(new Date());
        return person;
    }
    
    private long getPersonsCount() {
        try {
            Query countQuery = entityManager.createQuery("select count(p.id) from Person p");
            return (long) countQuery.getSingleResult();
        } catch (javax.persistence.PersistenceException ex) {
            System.out.println("Type not found exception - new database?");
            return 0;
        }
    }
}
